#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""A single HTTP request represented as a subclass of ``testtools.TestCase``

The test case encapsulates the request headers and body and expected
response headers and body. When the test is run an HTTP request is
made using urllib3. Assertions are made against the response.
"""

from collections import OrderedDict
import copy
import functools
import os
import re
import sys
from testtools import testcase
import time
from unittest import result

import six
from six.moves import http_cookies
from six.moves.urllib import parse as urlparse
import testtools
import wsgi_intercept

from gabbi import __version__
from gabbi import exception
from gabbi.handlers import base
from gabbi import utils


MAX_CHARS_OUTPUT = 2000

REPLACERS = [
    'SCHEME',
    'NETLOC',
    'ENVIRON',
    'LOCATION',
    'COOKIE',
    'LAST_URL',
    'URL',
    'HEADERS',
    'RESPONSE',
]

# Basic test template determining both valid keys and default values
BASE_TEST = {
    'name': '',
    'desc': '',
    'verbose': False,
    'ssl': False,
    'redirects': False,
    'method': 'GET',
    'url': '',
    'status': '200',
    'request_headers': {},
    'query_parameters': {},
    'data': '',
    'xfail': False,
    'skip': '',
    'poll': {},
    'use_prior_test': True,
}


def potentialFailure(func):
    """Decorate a test method that is expected to fail if 'xfail' is true."""
    @functools.wraps(func)
    def wrapper(self):
        if self.test_data['xfail']:
            try:
                func(self)
            except Exception:
                if hasattr(testcase, '_ExpectedFailure'):
                    raise testcase._ExpectedFailure(sys.exc_info())
                else:
                    self._addExpectedFailure(self.result, sys.exc_info())
            else:
                if hasattr(self, '_addUnexpectedSuccess'):
                    self._addUnexpectedSuccess(self.result)
                else:
                    raise testcase._UnexpectedSuccess
        else:
            func(self)
    return wrapper


class HTTPTestCase(testtools.TestCase):
    """Encapsulate a single HTTP request as a TestCase.

    If the test is a member of a sequence of requests, ensure that prior
    tests are run.

    To keep the test harness happy we need to make sure the setUp and
    tearDown are only run once.
    """

    base_test = copy.copy(BASE_TEST)

    def setUp(self):
        if not self.has_run:
            super(HTTPTestCase, self).setUp()
            for fixture in self.inner_fixtures:
                self.useFixture(fixture())

    def tearDown(self):
        if not self.has_run:
            super(HTTPTestCase, self).tearDown()
        self.has_run = True

    def run(self, result=None):
        """Store the current result handler on this test."""
        self.result = result
        super(HTTPTestCase, self).run(result)

    @potentialFailure
    def test_request(self):
        """Run this request if it has not yet run.

        If there is a prior test in the sequence, run it first.
        """
        if self.has_run:
            return

        if self.test_data['skip']:
            self.skipTest(self.test_data['skip'])

        if (self.prior and not self.prior.has_run and
                self.test_data['use_prior_test']):
            # Use a different result so we don't count this test
            # in the results.
            self.prior.run(result.TestResult())
        self._run_test()

    def get_content_handler(self, content_type):
        """Determine the content handler for this media type."""
        for handler in self.content_handlers:
            if handler.accepts(content_type):
                return handler
        return None

    def replace_template(self, message, escape_regex=False):
        """Replace magic strings in message."""
        if isinstance(message, dict):
            for k in message:
                message[k] = self.replace_template(message[k],
                                                   escape_regex=escape_regex)
            return message

        if isinstance(message, list):
            return [self.replace_template(line, escape_regex=escape_regex)
                    for line in message]

        for replacer in REPLACERS:
            template = '$%s' % replacer
            method = '_%s_replace' % replacer.lower()
            try:
                if template in message:
                    try:
                        replace = getattr(self, method)
                        message = replace(message, escape_regex=escape_regex)
                    except (KeyError, AttributeError, ValueError) as exc:
                        raise AssertionError(
                            'unable to replace %s in %s, data unavailable: %s'
                            % (template, message, exc))
            except TypeError:
                # Message is not a string
                pass

        return message

    def load_data_file(self, filename):
        """Read a file from the current test directory."""
        return self._load_data_file(filename)

    def _assert_response(self):
        """Compare the response with expected data."""
        self._test_status(self.test_data['status'], self.response['status'])

        for handler in self.response_handlers:
            handler(self)

    def _clean_query_value(self, value):
        """Clean up a single query from query_parameters."""
        value = self.replace_template(value)
        # stringify ints in Python version independent fashion
        value = '%s' % value
        value = value.encode('UTF-8')
        return value

    @staticmethod
    def _regex_replacer(replacer, escape_regex):
        """Wrap a replacer function to escape return values in a regex."""
        if escape_regex:
            @functools.wraps(replacer)
            def replace(match):
                return re.escape(replacer(match))

            return replace
        else:
            return replacer

    def _environ_replace(self, message, escape_regex=False):
        """Replace an indicator in a message with the environment value.

        If value can be a number, cast it as such. If value is a form of
        "null", "true", or "false" cast it to None, True, False.
        """
        value = re.sub(self._replacer_regex('ENVIRON'),
                       self._regex_replacer(self._environ_replacer,
                                            escape_regex),
                       message)
        try:
            if '.' in value:
                value = float(value)
            else:
                value = int(value)
            return value
        except ValueError:
            pass
        if value.lower() == "false":
            return False
        if value.lower() == "true":
            return True
        if value.lower() == "null":
            return None
        return value

    @staticmethod
    def _environ_replacer(match):
        """Replace a regex match with an environment value.

        Let KeyError raise if variable not present.
        """
        environ_name = match.group('arg')
        return os.environ[environ_name]

    def _cookie_replace(self, message, escape_regex=False):
        """Replace $COOKIE in a message.

        With cookie data from set-cookie in the prior request.
        """
        return re.sub(self._simple_replacer_regex('COOKIE'),
                      self._regex_replacer(self._cookie_replacer,
                                           escape_regex),
                      message)

    def _cookie_replacer(self, match):
        """Replace a regex match with the cookie of a previous response."""
        case = match.group('case')
        if case:
            referred_case = self.history[case]
        else:
            referred_case = self.prior
        response_cookies = referred_case.response['set-cookie']
        cookies = http_cookies.SimpleCookie()
        cookies.load(response_cookies)
        cookie_string = cookies.output(attrs=[], header='', sep=',').strip()
        return cookie_string

    def _headers_replace(self, message, escape_regex=False):
        """Replace a header indicator in a message with that headers value from
        the prior request.
        """
        return re.sub(self._replacer_regex('HEADERS'),
                      self._regex_replacer(self._header_replacer,
                                           escape_regex),
                      message)

    def _header_replacer(self, match):
        """Replace a regex match with the value of a prior header."""
        header_key = match.group('arg')
        case = match.group('case')
        if case:
            referred_case = self.history[case]
        else:
            referred_case = self.prior
        return referred_case.response[header_key.lower()]

    def _last_url_replace(self, message, escape_regex=False):
        """Replace $LAST_URL in a message.

        With the URL used in the prior request.
        """
        last_url = self.prior.url
        if escape_regex:
            last_url = re.escape(last_url)
        return message.replace('$LAST_URL', last_url)

    def _url_replace(self, message, escape_regex=False):
        """Replace $URL in a message.

        With the URL used in a previous request.
        """
        return re.sub(self._simple_replacer_regex('URL'),
                      self._regex_replacer(self._url_replacer,
                                           escape_regex),
                      message)

    def _url_replacer(self, match):
        """Replace a regex match with the value of a previous url."""
        case = match.group('case')
        if case:
            referred_case = self.history[case]
        else:
            referred_case = self.prior
        return referred_case.url

    def _location_replace(self, message, escape_regex=False):
        """Replace $LOCATION in a message.

        With the location header from a previous request.
        """
        return re.sub(self._simple_replacer_regex('LOCATION'),
                      self._regex_replacer(self._location_replacer,
                                           escape_regex),
                      message)

    def _location_replacer(self, match):
        """Replace a regex match with the value of a previous location."""
        case = match.group('case')
        if case:
            referred_case = self.history[case]
        else:
            referred_case = self.prior
        return referred_case.location

    def _load_data_file(self, filename):
        """Read a file from the current test directory."""
        path = os.path.join(self.test_directory, filename)
        has_dir_traversal = os.path.relpath(
            path, start=self.test_directory).startswith(os.pardir)
        if has_dir_traversal:
            raise ValueError(
                'Attempted loading of data file outside test directory: %s'
                % filename)
        with open(path, mode='rb') as data_file:
            return data_file.read()

    def _netloc_replace(self, message, escape_regex=False):
        """Replace $NETLOC with the current host and port."""
        netloc = self.netloc
        if self.prefix:
            netloc = '%s%s' % (netloc, self.prefix)
        if escape_regex:
            netloc = re.escape(netloc)
        return message.replace('$NETLOC', netloc)

    def _parse_url(self, url):
        """Create a url from test data.

        If provided with a full URL, just return that. If SSL is requested
        set the scheme appropriately.

        Scheme and netloc are saved for later use in comparisons.
        """
        query_params = self.test_data['query_parameters']
        ssl = self.test_data['ssl']

        parsed_url = urlparse.urlsplit(url)
        if not parsed_url.scheme:
            full_url = utils.create_url(url, self.host, port=self.port,
                                        prefix=self.prefix, ssl=ssl)
            # parse again to set updated netloc and scheme
            parsed_url = urlparse.urlsplit(full_url)

        self.scheme = parsed_url.scheme
        self.netloc = parsed_url.netloc

        if query_params:
            query_string = self._update_query_params(parsed_url.query,
                                                     query_params)
        else:
            query_string = parsed_url.query

        return urlparse.urlunsplit((parsed_url.scheme, parsed_url.netloc,
                                    parsed_url.path, query_string, ''))

    _history_regex = (
        r"(?:\$HISTORY\[(?P<quote1>['\"])(?P<case>.+?)(?P=quote1)\]\.)??"
    )

    @staticmethod
    def _replacer_regex(key):
        """Compose a regular expression for test template variables."""
        case = HTTPTestCase._history_regex
        return r"%s\$%s\[(?P<quote>['\"])(?P<arg>.+?)(?P=quote)\]" % (
            case, key)

    @staticmethod
    def _simple_replacer_regex(key):
        """Compose a regular expression for simple variable replacement."""
        case = HTTPTestCase._history_regex
        return r"%s\$%s" % (case, key)

    def _response_replace(self, message, escape_regex=False):
        """Replace a content path with the value from a previous response.

        If the match would replace the entire message, then don't cast it
        to a string.
        """
        regex = self._replacer_regex('RESPONSE')
        match = re.match('^%s$' % regex, message)
        if match:
            return self._response_replacer(match, preserve=True)
        return re.sub(regex,
                      self._regex_replacer(self._response_replacer,
                                           escape_regex),
                      message)

    def _response_replacer(self, match, preserve=False):
        """Replace a regex match with the value from a previous response."""
        response_path = match.group('arg')
        case = match.group('case')
        if case:
            referred_case = self.history[case]
        else:
            referred_case = self.prior
        replacer_class = self.get_content_handler(
            referred_case.response.get('content-type'))
        # If no handler can be found use the null replacer,
        # which returns "foo" when "$RESPONSE['foo']".
        replacer_class = replacer_class or base.ContentHandler
        result = replacer_class.replacer(
            referred_case.response_data, response_path)
        if preserve:
            return result
        else:
            return six.text_type(result)

    def _run_request(self, url, method, headers, body, redirect=False):
        """Run the http request and decode output.

        The call to make the request will catch a WSGIAppError from
        wsgi_intercept so that the real traceback from a catastrophic
        error in the intercepted app can be examined.
        """

        if 'user-agent' not in (key.lower() for key in headers):
            headers['user-agent'] = "gabbi/%s (Python urllib3)" % __version__

        try:
            response, content = self.http.request(
                url,
                method=method,
                headers=headers,
                body=body,
                redirect=redirect
            )
        except wsgi_intercept.WSGIAppError as exc:
            # Extract and re-raise the wrapped exception.
            six.reraise(exc.exception_type, exc.exception_value,
                        exc.traceback)

        # Set headers and location attributes for follow on requests
        self.response = response
        if 'location' in response:
            self.location = response['location']

        # Decode and store response
        decoded_output = utils.decode_response_content(response, content)
        self.content_type = response.get('content-type', '').lower()
        loader_class = self.get_content_handler(self.content_type)
        if decoded_output and loader_class:
            # save structured response data
            self.response_data = loader_class.loads(decoded_output)
        else:
            self.response_data = None
        self.output = decoded_output

    def _run_test(self):
        """Make an HTTP request and compare the response with expectations."""
        test = self.test_data

        base_url = self.replace_template(test['url'])
        # Save the URL after replacers but before query_parameters
        self.url = base_url
        full_url = self._parse_url(base_url)

        method = test['method'].upper()
        headers = test['request_headers']
        for name in headers:
            try:
                headers[name] = self.replace_template(headers[name])
            except TypeError as exc:
                raise exception.GabbiFormatError(
                    'malformed headers in test %s: %s' % (test['name'], exc))

        if test['data'] != '':
            body = self._test_data_to_string(
                test['data'],
                utils.extract_content_type(headers, default='')[0])
        else:
            body = ''

        # ensure body is bytes, encoding as UTF-8 because that's
        # what we do here
        if isinstance(body, six.text_type):
            body = body.encode('UTF-8')

        if test['poll']:
            count = int(float(self.replace_template(
                test['poll'].get('count', 1))))
            delay = float(self.replace_template(test['poll'].get('delay', 1)))
            failure = None
            while count:
                try:
                    self._run_request(full_url, method, headers, body,
                                      redirect=test['redirects'])
                    self._assert_response()
                    failure = None
                    break
                except (AssertionError, utils.ConnectionRefused) as exc:
                    failure = exc

                count -= 1
                time.sleep(delay)

            if failure:
                raise failure
        else:
            self._run_request(full_url, method, headers, body,
                              redirect=test['redirects'])
            self._assert_response()

    def _scheme_replace(self, message, escape_regex=False):
        """Replace $SCHEME with the current protocol."""
        scheme = re.escape(self.scheme) if escape_regex else self.scheme
        return message.replace('$SCHEME', scheme)

    def _test_data_to_string(self, data, content_type):
        """Turn the request data into a string.

        If the data is not binary, replace template strings.
        """
        if isinstance(data, str):
            if data.startswith('<@'):
                info = self.load_data_file(data.replace('<@', '', 1))
                if utils.not_binary(content_type):
                    data = six.text_type(info, 'UTF-8')
                else:
                    return info
        else:
            dumper_class = self.get_content_handler(content_type)
            if dumper_class:
                data = self.replace_template(data)
                data = dumper_class.dumps(data, test=self)
            else:
                raise ValueError(
                    'unable to process data to %s' % content_type)
        return self.replace_template(data)

    def _test_status(self, expected_status, observed_status):
        """Confirm we got the expected status.

        If the status contains one or more || then it is treated as a
        list of acceptable statuses.
        """
        expected_status = str(expected_status)
        if '||' in expected_status:
            statii = [stat.strip() for stat in expected_status.split('||')]
        else:
            statii = [expected_status.strip()]

        self.assert_in_or_print_output(observed_status, statii)

    def _update_query_params(self, original_query_string, query_params):
        """Update a query string from query_params dict.

        An OrderedDict is used to allow easier testing and greater
        predictability when doing query updates.
        """
        encoded_query_params = OrderedDict()

        for param, value in query_params.items():
            # isinstance used because we can iter a string
            if isinstance(value, list):
                encoded_query_params[param] = [
                    self._clean_query_value(subvalue)
                    for subvalue in value]
            else:
                encoded_query_params[param] = (
                    self._clean_query_value(value))

        query_string = urlparse.urlencode(
            encoded_query_params, doseq=True)
        if original_query_string:
            query_string = '&'.join([original_query_string, query_string])

        return query_string

    def assert_in_or_print_output(self, expected, iterable):
        """Assert the iterable contains expected or print some output.

        If the output is long, it is limited by either GABBI_MAX_CHARS_OUTPUT
        in the environment or the MAX_CHARS_OUTPUT constant.
        """
        if utils.not_binary(utils.parse_content_type(self.content_type)[0]):
            if expected in iterable:
                return

            if self.response_data:
                dumper_class = self.get_content_handler(self.content_type)
                if dumper_class:
                    full_response = dumper_class.dumps(self.response_data,
                                                       pretty=True, test=self)
                else:
                    full_response = self.output
            else:
                full_response = self.output

            max_chars = os.getenv('GABBI_MAX_CHARS_OUTPUT', MAX_CHARS_OUTPUT)
            response = full_response[0:max_chars]
            is_truncated = (len(response) != len(full_response))

            if iterable == self.output:
                msg = "'%s' not found in %s%s" % (
                    expected, response,
                    '\n...truncated...' if is_truncated else ''
                )
            else:
                msg = "'%s' not found in %s, %sresponse:\n%s" % (
                    expected, iterable,
                    'truncated ' if is_truncated else '',
                    response)
            self.fail(msg)
        else:
            self.assertIn(expected, iterable)



"""A sample pytest module for pytest >= 3.0."""

# For pathname munging
import os

# The module that py_test_generator comes from.
from gabbi import driver

# We need test_pytest so that pytest test collection works properly.
# Without this, the pytest_generate_tests method below will not be
# called.
from gabbi.driver import test_pytest  # noqa

# We need access to the WSGI application that hosts our service
from myapp import wsgiapp

# We're using fixtures in the YAML files, we need to know where to
# load them from.
from myapp.test import fixtures

# By convention the YAML files are put in a directory named
# "gabbits" that is in the same directory as the Python test file.
TESTS_DIR = 'gabbits'


def pytest_generate_tests(metafunc):
    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
    driver.py_test_generator(
        test_dir, intercept=wsgiapp.app,
        fixture_module=fixtures, metafunc=metafunc)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""A place to put test of the replacers.
"""

import os

import unittest

from gabbi import case


class EnvironReplaceTest(unittest.TestCase):

    def test_environ_boolean(self):
        """Environment variables are always strings

        That doesn't always suit our purposes, so test that "True"
        and "False" become booleans as a special case.
        """
        http_case = case.HTTPTestCase('test_request')
        message = "$ENVIRON['moo']"

        os.environ['moo'] = "True"
        self.assertEqual(True, http_case._environ_replace(message))

        os.environ['moo'] = "False"
        self.assertEqual(False, http_case._environ_replace(message))

        os.environ['moo'] = "true"
        self.assertEqual(True, http_case._environ_replace(message))

        os.environ['moo'] = "faLse"
        self.assertEqual(False, http_case._environ_replace(message))

        os.environ['moo'] = "null"
        self.assertEqual(None, http_case._environ_replace(message))

        os.environ['moo'] = "1"
        self.assertEqual(1, http_case._environ_replace(message))

        os.environ['moo'] = "cow"
        self.assertEqual("cow", http_case._environ_replace(message))

        message = '$ENVIRON["moo"]'

        os.environ['moo'] = "True"
        self.assertEqual(True, http_case._environ_replace(message))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Keep one single global jsonpath parser."""

from jsonpath_rw_ext import parser


PARSER = None


def parse(path):
    """Parse a JSONPath expression use the global parser."""
    global PARSER
    if not PARSER:
        PARSER = parser.ExtentedJsonPathParser()
    return PARSER.parse(path)



"""A sample test module."""

# For pathname munging
import os

# The module that build_tests comes from.
from gabbi import driver

# We need access to the WSGI application that hosts our service
from myapp import wsgiapp


# We're using fixtures in the YAML files, we need to know where to
# load them from.
from myapp.test import fixtures

# By convention the YAML files are put in a directory named
# "gabbits" that is in the same directory as the Python test file.
TESTS_DIR = 'gabbits'


def load_tests(loader, tests, pattern):
    """Provide a TestSuite to the discovery process."""
    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
    # Pass "require_ssl=True" as an argument to force all tests
    # to use SSL in requests.
    return driver.build_tests(test_dir, loader,
                              intercept=wsgiapp.app,
                              fixture_module=fixtures)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""A place to put tests of URL parsing.

These verbosely cover the _parse_url method to make sure it
behaves.
"""

from collections import OrderedDict
import copy
import unittest
import uuid

from gabbi import case


class UrlParseTest(unittest.TestCase):

    @staticmethod
    def make_test_case(host, port=8000, prefix='', ssl=False, params=None):
        # Attributes used are port, prefix and host and they must
        # be set manually here, due to metaclass magics elsewhere.
        # test_data must have a base value.
        http_case = case.HTTPTestCase('test_request')
        http_case.test_data = copy.copy(case.BASE_TEST)
        http_case.host = host
        http_case.port = port
        http_case.prefix = prefix
        http_case.test_data['ssl'] = ssl
        http_case.test_data['query_parameters'] = params or {}
        return http_case

    def test_parse_url(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('http://%s:8000/foobar' % host, parsed_url)

    def test_parse_prefix(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host, prefix='/noise')
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('http://%s:8000/noise/foobar' % host, parsed_url)

    def test_parse_full(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host)
        parsed_url = http_case._parse_url('http://example.com/house')

        self.assertEqual('http://example.com/house', parsed_url)

    def test_with_ssl(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host, ssl=True)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('https://%s:8000/foobar' % host, parsed_url)

    def test_default_port_http(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host, port='80')
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('http://%s/foobar' % host, parsed_url)

    def test_default_port_int(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host, port=80)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('http://%s/foobar' % host, parsed_url)

    def test_default_port_https(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host, port='443', ssl=True)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('https://%s/foobar' % host, parsed_url)

    def test_default_port_https_no_ssl(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host, port='443')
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('http://%s:443/foobar' % host, parsed_url)

    def test_https_port_80_ssl(self):
        host = uuid.uuid4().hex
        http_case = self.make_test_case(host, port='80', ssl=True)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('https://%s:80/foobar' % host, parsed_url)

    def test_ipv6_url(self):
        host = '::1'
        http_case = self.make_test_case(host, port='80', ssl=True)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('https://[%s]:80/foobar' % host, parsed_url)

    def test_ipv6_full_url(self):
        host = '::1'
        http_case = self.make_test_case(host, port='80', ssl=True)
        parsed_url = http_case._parse_url(
            'http://[2001:4860:4860::8888]/foobar')

        self.assertEqual('http://[2001:4860:4860::8888]/foobar', parsed_url)

    def test_ipv6_no_double_colon_wacky_ssl(self):
        host = 'FEDC:BA98:7654:3210:FEDC:BA98:7654:3210'
        http_case = self.make_test_case(host, port='80', ssl=True)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual(
            'https://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:80/foobar',
            parsed_url)

        http_case = self.make_test_case(host, ssl=True)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual(
            'https://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:8000/foobar',
            parsed_url)

    def test_add_query_params(self):
        host = uuid.uuid4().hex
        # Use a sequence of tuples to ensure order.
        query = OrderedDict([('x', 1), ('y', 2)])
        http_case = self.make_test_case(host, params=query)
        parsed_url = http_case._parse_url('/foobar')

        self.assertEqual('http://%s:8000/foobar?x=1&y=2' % host, parsed_url)

    def test_extend_query_params(self):
        host = uuid.uuid4().hex
        # Use a sequence of tuples to ensure order.
        query = OrderedDict([('x', 1), ('y', 2)])
        http_case = self.make_test_case(host, params=query)
        parsed_url = http_case._parse_url('/foobar?alpha=beta')

        self.assertEqual('http://%s:8000/foobar?alpha=beta&x=1&y=2'
                         % host, parsed_url)

    def test_extend_query_params_full_url(self):
        host = 'stub'
        query = OrderedDict([('x', 1), ('y', 2)])
        http_case = self.make_test_case(host, params=query)
        parsed_url = http_case._parse_url(
            'http://example.com/foobar?alpha=beta')

        self.assertEqual('http://example.com/foobar?alpha=beta&x=1&y=2',
                         parsed_url)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test response handlers.
"""

import json
import unittest

from gabbi import case
from gabbi.exception import GabbiFormatError
from gabbi.handlers import core
from gabbi.handlers import jsonhandler
from gabbi import suitemaker


class HandlersTest(unittest.TestCase):
    """Test the response handlers.

    Note that this does not test the magic template variables, that
    should be tested somewhere else.
    """

    def setUp(self):
        super(HandlersTest, self).setUp()
        self.test_class = case.HTTPTestCase
        self.test = suitemaker.TestBuilder('mytest', (self.test_class,),
                                           {'test_data': {},
                                           'content_handlers': []})

    def test_empty_response_handler(self):
        self.test.test_data = {'url': '$RESPONSE["barnabas"]'}
        self.test.response = {'content-type': 'unmatchable'}
        self.test.response_data = ''
        self.test.prior = self.test

        url = self.test('test_request').replace_template(
            self.test.test_data['url'])
        self.assertEqual('barnabas', url)

        self.test.response_data = None
        self.test.content_handlers = [jsonhandler.JSONHandler()]
        url = self.test('test_request').replace_template(
            self.test.test_data['url'])
        self.assertEqual('barnabas', url)

    def test_response_strings(self):
        handler = core.StringResponseHandler()
        self.test.content_type = "text/plain"
        self.test.response_data = None
        self.test.test_data = {'response_strings': ['alpha', 'beta']}
        self.test.output = 'alpha\nbeta\n'
        self._assert_handler(handler)

    def test_response_strings_fail(self):
        handler = core.StringResponseHandler()
        self.test.content_type = "text/plain"
        self.test.response_data = None
        self.test.test_data = {'response_strings': ['alpha', 'beta']}
        self.test.output = 'alpha\nbta\n'
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_strings_fail_big_output(self):
        handler = core.StringResponseHandler()
        self.test.content_type = "text/plain"
        self.test.response_data = None
        self.test.test_data = {'response_strings': ['alpha', 'beta']}
        self.test.output = 'alpha\nbta\n' * 1000
        with self.assertRaises(AssertionError) as cm:
            self._assert_handler(handler)

        msg = str(cm.exception)
        self.assertEqual(2036, len(msg))

    def test_response_strings_fail_big_payload(self):
        string_handler = core.StringResponseHandler()
        # Register the JSON handler so response_data is set.
        json_handler = jsonhandler.JSONHandler()
        self.test.response_handlers = [string_handler, json_handler]
        self.test.content_handlers = [json_handler]
        self.test.content_type = "application/json"
        self.test.test_data = {'response_strings': ['foobar']}
        self.test.response_data = {
            'objects': [{'name': 'cw',
                         'location': 'barn'},
                        {'name': 'chris',
                         'location': 'house'}] * 100
        }
        self.test.output = json.dumps(self.test.response_data)
        with self.assertRaises(AssertionError) as cm:
            self._assert_handler(string_handler)

        msg = str(cm.exception)
        self.assertEqual(2038, len(msg))
        # Check the pprint of the json
        self.assertIn('      "location": "house"', msg)

    def test_response_string_list_type(self):
        handler = core.StringResponseHandler()
        self.test.test_data = {
            'name': 'omega test',
            'response_strings': 'omega'
        }
        self.test.output = 'omega\n'
        with self.assertRaises(GabbiFormatError) as exc:
            self._assert_handler(handler)
            self.assertIn('has incorrect type', str(exc))
            self.assertIn("response_strings in 'omega test'",
                          str(exc))

    def test_response_json_paths(self):
        handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.objects[0].name': 'cow',
            '$.objects[1].location': 'house',
        }}
        self.test.response_data = {
            'objects': [{'name': 'cow',
                         'location': 'barn'},
                        {'name': 'chris',
                         'location': 'house'}]
        }
        self._assert_handler(handler)

    def test_response_json_paths_fail_data(self):
        handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.objects[0].name': 'cow',
            '$.objects[1].location': 'house',
        }}
        self.test.response_data = {
            'objects': [{'name': 'cw',
                         'location': 'barn'},
                        {'name': 'chris',
                         'location': 'house'}]
        }
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_json_paths_fail_path(self):
        handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.objects[1].name': 'cow',
        }}
        self.test.response_data = {
            'objects': [{'name': 'cow',
                         'location': 'barn'},
                        {'name': 'chris',
                         'location': 'house'}]
        }
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_json_paths_regex(self):
        handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.objects[0].name': '/ow/',
        }}
        self.test.response_data = {
            'objects': [{'name': u'cow\U0001F404',
                         'location': 'barn'},
                        {'name': 'chris',
                         'location': 'house'}]
        }
        self._assert_handler(handler)

    def test_response_json_paths_regex_path_match(self):
        handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.pathtest': '//bar//',
        }}
        self.test.response_data = {
            'pathtest': '/foo/bar/baz'
        }
        self._assert_handler(handler)

    def test_response_json_paths_regex_path_nomatch(self):
        handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.pathtest': '//bar//',
        }}
        self.test.response_data = {
            'pathtest': '/foo/foobar/baz'
        }
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_json_paths_substitution_regex(self):
        handler = jsonhandler.JSONHandler()
        self.test.location = '/foo/bar'
        self.test.prior = self.test
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.pathtest': '/$LOCATION/',
        }}
        self.test.response_data = {
            'pathtest': '/foo/bar/baz'
        }
        self._assert_handler(handler)

    def test_response_json_paths_substitution_noregex(self):
        handler = jsonhandler.JSONHandler()
        self.test.location = '/foo/bar/'
        self.test.prior = self.test
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.pathtest': '$LOCATION',
        }}
        self.test.response_data = {
            'pathtest': '/foo/bar/baz'
        }
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_json_paths_substitution_esc_regex(self):
        handler = jsonhandler.JSONHandler()
        self.test.location = '/foo/bar?query'
        self.test.prior = self.test
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.pathtest': '/$LOCATION/',
        }}
        self.test.response_data = {
            'pathtest': '/foo/bar?query=value'
        }
        self._assert_handler(handler)

    def test_response_json_paths_regex_number(self):
        handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.test_data = {'response_json_paths': {
            '$.objects[0].name': '/\d+/',
        }}
        self.test.response_data = {
            'objects': [{'name': 99,
                         'location': 'barn'},
                        {'name': 'chris',
                         'location': 'house'}]
        }
        self._assert_handler(handler)

    def test_response_json_paths_dict_type(self):
        handler = jsonhandler.JSONHandler()
        self.test.test_data = {
            'name': 'omega test',
            'response_json_paths': ['alpha', 'beta']
        }
        self.test.output = 'omega\n'
        with self.assertRaises(GabbiFormatError) as exc:
            self._assert_handler(handler)
            self.assertIn('has incorrect type', str(exc))
            self.assertIn("response_json_paths in 'omega test'",
                          str(exc))

    def test_response_headers(self):
        handler = core.HeadersResponseHandler()
        self.test.response = {'content-type': 'text/plain'}

        self.test.test_data = {'response_headers': {
            'content-type': 'text/plain',
        }}
        self._assert_handler(handler)

        self.test.test_data = {'response_headers': {
            'Content-Type': 'text/plain',
        }}
        self._assert_handler(handler)

    def test_response_headers_regex(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'content-type': '/text/plain/',
        }}
        self.test.response = {'content-type': 'text/plain; charset=UTF-8'}
        self._assert_handler(handler)

    def test_response_headers_substitute_noregex(self):
        handler = core.HeadersResponseHandler()
        self.test.location = '/foo/bar/'
        self.test.prior = self.test
        self.test.test_data = {'response_headers': {
            'location': '$LOCATION',
        }}
        self.test.response = {'location': '/foo/bar/baz'}
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_headers_substitute_regex(self):
        handler = core.HeadersResponseHandler()
        self.test.location = '/foo/bar/'
        self.test.prior = self.test
        self.test.test_data = {'response_headers': {
            'location': '/^$LOCATION/',
        }}
        self.test.response = {'location': '/foo/bar/baz'}
        self._assert_handler(handler)

    def test_response_headers_substitute_esc_regex(self):
        handler = core.HeadersResponseHandler()
        self.test.scheme = 'git+ssh'
        self.test.test_data = {'response_headers': {
            'location': '/^$SCHEME://.*/',
        }}
        self.test.response = {'location': 'git+ssh://example.test'}
        self._assert_handler(handler)

    def test_response_headers_noregex_path_match(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'location': '/',
        }}
        self.test.response = {'location': '/'}
        self._assert_handler(handler)

    def test_response_headers_noregex_path_nomatch(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'location': '/',
        }}
        self.test.response = {'location': '/foo'}
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_headers_regex_path_match(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'location': '//bar//',
        }}
        self.test.response = {'location': '/foo/bar/baz'}
        self._assert_handler(handler)

    def test_response_headers_regex_path_nomatch(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'location': '//bar//',
        }}
        self.test.response = {'location': '/foo/foobar/baz'}
        with self.assertRaises(AssertionError):
            self._assert_handler(handler)

    def test_response_headers_fail_data(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'content-type': 'text/plain',
        }}
        self.test.response = {'content-type': 'application/json'}
        with self.assertRaises(AssertionError) as failure:
            self._assert_handler(handler)
        self.assertIn("Expect header content-type with value text/plain,"
                      " got application/json",
                      str(failure.exception))

    def test_response_headers_fail_header(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'location': '/somewhere',
        }}
        self.test.response = {'content-type': 'application/json'}
        with self.assertRaises(AssertionError) as failure:
            self._assert_handler(handler)
        self.assertIn("'location' header not present in response:",
                      str(failure.exception))

    def test_resonse_headers_stringify(self):
        handler = core.HeadersResponseHandler()
        self.test.test_data = {'response_headers': {
            'x-alpha-beta': 2.0,
        }}
        self.test.response = {'x-alpha-beta': '2.0'}
        self._assert_handler(handler)

        self.test.response = {'x-alpha-beta': 2.0}
        self._assert_handler(handler)

    def _assert_handler(self, handler):
        # Instantiate our contained test class by naming its test
        # method and then run its tests to confirm.
        test = self.test('test_request')
        handler(test)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Generate HTTP tests from YAML files

Each HTTP request is its own TestCase and can be requested to be run in
isolation from other tests. If it is a member of a sequence of requests,
prior requests will be run.

A sequence is represented by an ordered list in a single YAML file.

Each sequence becomes a TestSuite.

An entire directory of YAML files is a TestSuite of TestSuites.
"""

import glob
import inspect
import os
import unittest
from unittest import suite
import uuid
import warnings

from gabbi import exception
from gabbi import handlers
from gabbi import reporter
from gabbi import suitemaker
from gabbi import utils


def build_tests(path, loader, host=None, port=8001, intercept=None,
                test_loader_name=None, fixture_module=None,
                response_handlers=None, content_handlers=None,
                prefix='', require_ssl=False, url=None,
                inner_fixtures=None, verbose=False,
                use_prior_test=True):
    """Read YAML files from a directory to create tests.

    Each YAML file represents a list of HTTP requests.

    :param path: The directory where yaml files are located.
    :param loader: The TestLoader.
    :param host: The host to test against. Do not use with ``intercept``.
    :param port: The port to test against. Used with ``host``.
    :param intercept: WSGI app factory for wsgi-intercept.
    :param test_loader_name: Base name for test classes. Use this to align the
                             naming of the tests with other tests in a system.
    :param fixture_module: Python module containing fixture classes.
    :param response_handers: :class:`~gabbi.handlers.ResponseHandler` classes.
    :type response_handlers: List of ResponseHandler classes.
    :param content_handlers: ContentHandler classes.
    :type content_handlers: List of ContentHandler classes.
    :param prefix: A URL prefix for all URLs that are not fully qualified.
    :param url: A full URL to test against. Replaces host, port and prefix.
    :param require_ssl: If ``True``, make all tests default to using SSL.
    :param inner_fixtures: A list of ``Fixtures`` to use with each
                           individual test request.
    :param verbose: If ``True`` or ``'all'``, make tests verbose by default
                    ``'headers'`` and ``'body'`` are also accepted.
    :param use_prior_test: If ``True``, uses prior test to create ordered
                           sequence of tests
    :type inner_fixtures: List of fixtures.Fixture classes.
    :rtype: TestSuite containing multiple TestSuites (one for each YAML file).
    """

    # If url is being used, reset host, port and prefix.
    if url:
        host, port, prefix, force_ssl = utils.host_info_from_target(url)
        if force_ssl and not require_ssl:
            require_ssl = force_ssl

    # Exit immediately if we have no host to access, either via a real host
    # or an intercept.
    if not bool(host) ^ bool(intercept):
        raise AssertionError(
            'must specify exactly one of host or url, or intercept')

    # If the client has not provided a name to use as our base,
    # create one so that tests are effectively namespaced.
    if test_loader_name is None:
        all_test_base_name = inspect.stack()[1]
        all_test_base_name = os.path.splitext(
            os.path.basename(all_test_base_name[1]))[0]
    else:
        all_test_base_name = None

    # Initialize response and content handlers. This is effectively
    # duplication of effort but not results. This allows for
    # backwards compatibility for existing callers.
    response_handlers = response_handlers or []
    content_handlers = content_handlers or []
    handler_objects = []
    for handler in (content_handlers + response_handlers
                    + handlers.RESPONSE_HANDLERS):
        handler_objects.append(handler())

    top_suite = suite.TestSuite()
    for test_file in glob.iglob('%s/*.yaml' % path):
        if '_' in os.path.basename(test_file):
            warnings.warn(exception.GabbiSyntaxWarning(
                "'_' in test filename %s. This can break suite grouping."
                % test_file))
        if intercept:
            host = str(uuid.uuid4())
        suite_dict = utils.load_yaml(yaml_file=test_file)
        test_base_name = os.path.splitext(os.path.basename(test_file))[0]
        if all_test_base_name:
            test_base_name = '%s_%s' % (all_test_base_name, test_base_name)

        if require_ssl:
            if 'defaults' in suite_dict:
                suite_dict['defaults']['ssl'] = True
            else:
                suite_dict['defaults'] = {'ssl': True}

        if any((verbose == opt for opt in [True, 'all', 'headers', 'body'])):
            if 'defaults' in suite_dict:
                suite_dict['defaults']['verbose'] = verbose
            else:
                suite_dict['defaults'] = {'verbose': verbose}

        if not use_prior_test:
            if 'defaults' in suite_dict:
                suite_dict['defaults']['use_prior_test'] = use_prior_test
            else:
                suite_dict['defaults'] = {'use_prior_test': use_prior_test}

        file_suite = suitemaker.test_suite_from_dict(
            loader, test_base_name, suite_dict, path, host, port,
            fixture_module, intercept, prefix=prefix,
            test_loader_name=test_loader_name, handlers=handler_objects,
            inner_fixtures=inner_fixtures)
        top_suite.addTest(file_suite)
    return top_suite


def py_test_generator(test_dir, host=None, port=8001, intercept=None,
                      prefix=None, test_loader_name=None,
                      fixture_module=None, response_handlers=None,
                      content_handlers=None, require_ssl=False, url=None,
                      metafunc=None, use_prior_test=True):
    """Generate tests cases for py.test

    This uses build_tests to create TestCases and then yields them in
    a way that pytest can handle.
    """

    import pytest
    pluginmanager = pytest.config.pluginmanager
    pluginmanager.import_plugin('gabbi.pytester')

    loader = unittest.TestLoader()
    result = reporter.PyTestResult()
    tests = build_tests(test_dir, loader, host=host, port=port,
                        intercept=intercept,
                        test_loader_name=test_loader_name,
                        fixture_module=fixture_module,
                        response_handlers=response_handlers,
                        content_handlers=content_handlers,
                        prefix=prefix, require_ssl=require_ssl,
                        url=url, use_prior_test=use_prior_test)

    test_list = []
    for test in tests:
        if hasattr(test, '_tests'):
            # Establish fixtures as if they were tests. These will
            # be cleaned up by the pytester plugin.
            test_list.append(('start_%s' % test._tests[0].__class__.__name__,
                              test.start, result))
            for subtest in test:
                test_list.append(('%s' % subtest.__class__.__name__,
                                  subtest, result))
            test_list.append(('stop_%s' % test._tests[0].__class__.__name__,
                              test.stop))

    if metafunc:
        if metafunc.function == test_pytest:
            ids = []
            args = []
            for test in test_list:
                if len(test) >= 3:
                    name, method, arg = test
                else:
                    name, method = test
                    arg = None
                ids.append(name)
                args.append((method, arg))

            metafunc.parametrize("test, result", argvalues=args, ids=ids)
    else:
        # preserve backwards compatibility with old calling style
        return test_list


def test_pytest(test, result):
    if result:
        test(result)
    else:
        test()


def test_suite_from_yaml(loader, test_base_name, test_yaml, test_directory,
                         host, port, fixture_module, intercept, prefix=''):
    """Legacy wrapper retained for backwards compatibility."""

    with warnings.catch_warnings():  # ensures warnings filter is restored
        warnings.simplefilter('default', DeprecationWarning)
        warnings.warn('test_suite_from_yaml has been renamed to '
                      'test_suite_from_dict', DeprecationWarning, stacklevel=2)
    return suitemaker.test_suite_from_dict(
        loader, test_base_name, test_yaml, test_directory, host, port,
        fixture_module, intercept, prefix)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""A sample test module to exercise the code.

For the sake of exploratory development.
"""

import os
import sys

from gabbi import driver
from gabbi import fixture
from gabbi.handlers import base
from gabbi.tests import simple_wsgi
from gabbi.tests import util


TESTS_DIR = 'gabbits_intercept'


class TestFixtureOne(fixture.GabbiFixture):
    """Drive the fixture testing weakly."""
    pass


class TestFixtureTwo(fixture.GabbiFixture):
    """Drive the fixture testing weakly."""
    pass


class TestResponseHandler(base.ResponseHandler):
    """A sample response handler just to test."""

    test_key_suffix = 'test'
    test_key_value = []

    def preprocess(self, test):
        """Add some data if the data is a string."""
        try:
            test.output = test.output + '\nAnother line'
        except TypeError:
            pass

    def action(self, test, expected, value=None):
        expected = expected.replace('COW', '', 1)
        test.assertIn(expected, test.output)


# Incorporate the SkipAllFixture into this namespace so it can be used
# by tests (cf. skipall.yaml).
SkipAllFixture = fixture.SkipAllFixture


def load_tests(loader, tests, pattern):
    """Provide a TestSuite to the discovery process."""
    # Set and environment variable for one of the tests.
    util.set_test_environ()

    prefix = os.environ.get('GABBI_PREFIX')
    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
    return driver.build_tests(test_dir, loader, host=None,
                              intercept=simple_wsgi.SimpleWsgi,
                              test_loader_name=__name__,
                              prefix=prefix,
                              fixture_module=sys.modules[__name__],
                              response_handlers=[TestResponseHandler])



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Unit tests for the gabbi.suite.
"""

import sys
import unittest

from gabbi import fixture
from gabbi import suitemaker

VALUE_ERROR = 'value error sentinel'


class FakeFixture(fixture.GabbiFixture):

    def start_fixture(self):
        raise ValueError(VALUE_ERROR)


class SuiteTest(unittest.TestCase):

    def test_suite_catches_fixture_fail(self):
        """When a fixture fails in start_fixture it should fail
        the first test in the suite and skip the others.
        """
        loader = unittest.defaultTestLoader
        result = unittest.TestResult()
        test_data = {'fixtures': ['FakeFixture'],
                     'tests': [{'name': 'alpha', 'GET': '/'},
                               {'name': 'beta', 'GET': '/'}]}
        test_suite = suitemaker.test_suite_from_dict(
            loader, 'foo', test_data, '.', 'localhost',
            80, sys.modules[__name__], None)

        test_suite.run(result)

        self.assertEqual(2, len(result.skipped))
        self.assertEqual(1, len(result.errors))

        errored_test, trace = result.errors[0]

        self.assertIn('foo_alpha', str(errored_test))
        self.assertIn(VALUE_ERROR, trace)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test jsonpath handling
"""

import unittest

from gabbi.handlers import jsonhandler


extract = jsonhandler.JSONHandler.extract_json_path_value
nested_data = {
    'objects': [
        {'name': 'one', 'value': 'alpha'},
        {'name': 'two', 'value': 'beta'},
    ]
}
simple_list = {
    'objects': [
        'alpha',
        'gamma',
        'gabba',
        'hey',
        'carlton',
    ]
}


class JSONPathTest(unittest.TestCase):

    def test_basic_match(self):
        data = ['hi']
        match = extract(data, '$[0]')
        self.assertEqual('hi', match)

    def test_list_handling(self):
        data = ['hi', 'bye']
        match = extract(data, '$')
        self.assertEqual(data, match)

    def test_embedded_list_handling(self):
        match = extract(nested_data, '$.objects..name')
        self.assertEqual(['one', 'two'], match)

    def test_sorted_object_list(self):
        match = extract(nested_data, r'$.objects[\name][0].value')
        self.assertEqual('beta', match)

    def test_filtered_list(self):
        match = extract(nested_data, r'$.objects[?name = "one"].value')
        self.assertEqual('alpha', match)

    def test_sorted_simple_list(self):
        match = extract(simple_list, r'$.objects.`sorted`[-1]')
        self.assertEqual('hey', match)

    def test_len_simple_list(self):
        match = extract(simple_list, r'$.objects.`len`')
        self.assertEqual(5, match)

    def test_len_object_list(self):
        match = extract(nested_data, '$.objects.`len`')
        self.assertEqual(2, match)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test functions from the utils module.
"""

import unittest

from gabbi import utils


class BinaryTypesTest(unittest.TestCase):

    BINARY_TYPES = [
        'image/png',
        'application/binary',
    ]

    NON_BINARY_TYPES = [
        'text/plain',
        'application/atom+xml',
        'application/vnd.custom+json',
        'application/javascript',
        'application/json',
        'application/json-home',
        'application/xml',
    ]

    def test_not_binary(self):
        for media_type in self.NON_BINARY_TYPES:
            self.assertTrue(utils.not_binary(media_type),
                            '%s should not be binary' % media_type)

    def test_binary(self):
        for media_type in self.BINARY_TYPES:
            self.assertFalse(utils.not_binary(media_type),
                             '%s should be binary' % media_type)


class ParseContentTypeTest(unittest.TestCase):

    def test_parse_simple(self):
        self.assertEqual(
            ('text/plain', 'latin-1'),
            utils.parse_content_type('text/plain; charset=latin-1'))

    def test_parse_extra(self):
        self.assertEqual(
            ('text/plain', 'latin-1'),
            utils.parse_content_type(
                'text/plain; charset=latin-1; version=1.2'))

    def test_parse_default(self):
        self.assertEqual(
            ('text/plain', 'utf-8'),
            utils.parse_content_type('text/plain'))

    def test_parse_error_default(self):
        self.assertEqual(
            ('text/plain', 'utf-8'),
            utils.parse_content_type(
                'text/plain; face=ouch; charset=latin-1;'))

    def test_parse_nocharset_default(self):
        self.assertEqual(
            ('text/plain', 'utf-8'),
            utils.parse_content_type(
                'text/plain; face=ouch'))

    def test_parse_override_default(self):
        self.assertEqual(
            ('text/plain', 'latin-1'),
            utils.parse_content_type(
                'text/plain; face=ouch', default_charset='latin-1'))


class ExtractContentTypeTest(unittest.TestCase):

    def test_extract_content_type_default_both(self):
        """Empty dicts returns default type and chartset."""
        content_type, charset = utils.extract_content_type({})

        self.assertEqual('application/binary', content_type)
        self.assertEqual('utf-8', charset)

    def test_extract_content_type_default_charset(self):
        """Empty dicts returns default type and chartset."""
        content_type, charset = utils.extract_content_type({
            'content-type': 'text/colorful'})

        self.assertEqual('text/colorful', content_type)
        self.assertEqual('utf-8', charset)

    def test_extract_content_type_with_charset(self):
        content_type, charset = utils.extract_content_type(
            {'content-type': 'text/colorful; charset=latin-10'})

        self.assertEqual('text/colorful', content_type)
        self.assertEqual('latin-10', charset)

    def test_extract_content_type_multiple_params(self):
        content_type, charset = utils.extract_content_type(
            {'content-type': 'text/colorful; version=1.24; charset=latin-10'})

        self.assertEqual('text/colorful', content_type)
        self.assertEqual('latin-10', charset)

    def test_extract_content_type_bad_params(self):
        content_type, charset = utils.extract_content_type(
            {'content-type': 'text/colorful; version=1.24; charset=latin-10;'})

        self.assertEqual('text/colorful', content_type)
        self.assertEqual('utf-8', charset)


class ColorizeTest(unittest.TestCase):

    def test_colorize_missing_color(self):
        """Make sure that choosing a non-existent color is safe."""
        message = utils._colorize('CERULEAN', 'hello')
        self.assertEqual('hello', message)

        message = utils._colorize('BLUE', 'hello')
        self.assertNotEqual('hello', message)


class CreateURLTest(unittest.TestCase):

    def test_create_url_simple(self):
        url = utils.create_url('/foo/bar', 'test.host.com')
        self.assertEqual('http://test.host.com/foo/bar', url)

    def test_create_url_ssl(self):
        url = utils.create_url('/foo/bar', 'test.host.com', ssl=True)
        self.assertEqual('https://test.host.com/foo/bar', url)

    def test_create_url_prefix(self):
        url = utils.create_url('/foo/bar', 'test.host.com', prefix='/zoom')
        self.assertEqual('http://test.host.com/zoom/foo/bar', url)

    def test_create_url_port(self):
        url = utils.create_url('/foo/bar', 'test.host.com', port=8000)
        self.assertEqual('http://test.host.com:8000/foo/bar', url)

    def test_create_url_port_and_ssl(self):
        url = utils.create_url('/foo/bar', 'test.host.com', ssl=True,
                               port=8000)
        self.assertEqual('https://test.host.com:8000/foo/bar', url)

    def test_create_url_not_ssl_on_443(self):
        url = utils.create_url('/foo/bar', 'test.host.com', ssl=False,
                               port=443)
        self.assertEqual('http://test.host.com:443/foo/bar', url)

    def test_create_url_ssl_on_80(self):
        url = utils.create_url('/foo/bar', 'test.host.com', ssl=True,
                               port=80)
        self.assertEqual('https://test.host.com:80/foo/bar', url)

    def test_create_url_preserve_query(self):
        url = utils.create_url('/foo/bar?x=1&y=2', 'test.host.com', ssl=True,
                               port=80)
        self.assertEqual('https://test.host.com:80/foo/bar?x=1&y=2', url)

    def test_create_url_ipv6_ssl(self):
        url = utils.create_url('/foo/bar?x=1&y=2', '::1', ssl=True)
        self.assertEqual('https://[::1]/foo/bar?x=1&y=2', url)

    def test_create_url_ipv6_ssl_weird_port(self):
        url = utils.create_url('/foo/bar?x=1&y=2', '::1', ssl=True, port=80)
        self.assertEqual('https://[::1]:80/foo/bar?x=1&y=2', url)

    def test_create_url_ipv6_full(self):
        url = utils.create_url('/foo/bar?x=1&y=2',
                               '2607:f8b0:4000:801::200e', port=8080)
        self.assertEqual(
            'http://[2607:f8b0:4000:801::200e]:8080/foo/bar?x=1&y=2', url)

    def test_create_url_ipv6_already_bracket(self):
        url = utils.create_url(
            '/foo/bar?x=1&y=2', '[2607:f8b0:4000:801::200e]', port=999)
        self.assertEqual(
            'http://[2607:f8b0:4000:801::200e]:999/foo/bar?x=1&y=2', url)

    def test_create_url_no_double_colon(self):
        url = utils.create_url(
            '/foo', 'FEDC:BA98:7654:3210:FEDC:BA98:7654:3210', port=999)
        self.assertEqual(
            'http://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:999/foo', url)


class UtilsHostInfoFromTarget(unittest.TestCase):

    def _test_hostport(self, url_or_host, expected_host,
                       provided_prefix=None, expected_port=None,
                       expected_prefix=None, expected_ssl=False):
        host, port, prefix, ssl = utils.host_info_from_target(
            url_or_host, provided_prefix)

        # normalize hosts, they are case insensitive
        self.assertEqual(expected_host.lower(), host.lower())
        # port can be a string or int depending on the inputs
        self.assertEqual(expected_port, port)
        self.assertEqual(expected_prefix, prefix)
        self.assertEqual(expected_ssl, ssl)

    def test_plain_url_no_port(self):
        self._test_hostport('http://foobar.com/news',
                            'foobar.com',
                            expected_port=None,
                            expected_prefix='/news')

    def test_plain_url_with_port(self):
        self._test_hostport('http://foobar.com:80/news',
                            'foobar.com',
                            expected_port=80,
                            expected_prefix='/news')

    def test_ssl_url(self):
        self._test_hostport('https://foobar.com/news',
                            'foobar.com',
                            expected_prefix='/news',
                            expected_ssl=True)

    def test_ssl_port80_url(self):
        self._test_hostport('https://foobar.com:80/news',
                            'foobar.com',
                            expected_prefix='/news',
                            expected_port=80,
                            expected_ssl=True)

    def test_ssl_port_url(self):
        self._test_hostport('https://foobar.com:999/news',
                            'foobar.com',
                            expected_prefix='/news',
                            expected_port=999,
                            expected_ssl=True)

    def test_simple_hostport(self):
        self._test_hostport('foobar.com:999',
                            'foobar.com',
                            expected_port='999')

    def test_simple_hostport_with_prefix(self):
        self._test_hostport('foobar.com:999',
                            'foobar.com',
                            provided_prefix='/news',
                            expected_port='999',
                            expected_prefix='/news')

    def test_ipv6_url_long(self):
        self._test_hostport(
            'http://[FEDC:BA98:7654:3210:FEDC:BA98:7654:3210]:999/news',
            'FEDC:BA98:7654:3210:FEDC:BA98:7654:3210',
            expected_port=999,
            expected_prefix='/news')

    def test_ipv6_url_localhost(self):
        self._test_hostport(
            'http://[::1]:999/news',
            '::1',
            expected_port=999,
            expected_prefix='/news')

    def test_ipv6_host_localhost(self):
        # If a user wants to use the hostport form, then they need
        # to hack it with the brackets.
        self._test_hostport(
            '[::1]',
            '::1')

    def test_ipv6_hostport_localhost(self):
        self._test_hostport(
            '[::1]:999',
            '::1',
            expected_port='999')



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from gabbi import exception
from gabbi import suitemaker


class SuiteMakerTest(unittest.TestCase):

    def setUp(self):
        super(SuiteMakerTest, self).setUp()
        self.loader = unittest.defaultTestLoader

    def test_tests_key_required(self):
        test_yaml = {'name': 'house', 'url': '/'}

        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertEqual('malformed test file, "tests" key required',
                         str(failure.exception))

    def test_upper_dict_required(self):
        test_yaml = [{'name': 'house', 'url': '/'}]
        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertEqual('malformed test file, invalid format',
                         str(failure.exception))

    def test_inner_list_required(self):
        test_yaml = {'tests': {'name': 'house', 'url': '/'}}
        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertIn('test chunk is not a dict at',
                      str(failure.exception))

    def test_name_key_required(self):
        test_yaml = {'tests': [{'url': '/'}]}

        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertEqual('Test name missing in a test in foo.',
                         str(failure.exception))

    def test_url_key_required(self):
        test_yaml = {'tests': [{'name': 'missing url'}]}

        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertEqual('Test url missing in test foo_missing_url.',
                         str(failure.exception))

    def test_unsupported_key_errors(self):
        test_yaml = {'tests': [{
            'url': '/',
            'name': 'simple',
            'bad_key': 'wow',
        }]}

        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertIn("Invalid test keys used in test foo_simple:",
                      str(failure.exception))

    def test_method_url_pair_format_error(self):
        test_yaml = {'defaults': {'GET': '/foo'}, 'tests': []}
        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertIn('"METHOD: url" pairs not allowed in defaults',
                      str(failure.exception))

    def test_method_url_pair_duplication_format_error(self):
        test_yaml = {'tests': [{
            'GET': '/',
            'POST': '/',
            'name': 'duplicate methods',
        }]}
        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertIn(
            'duplicate method/URL directive in "foo_duplicate_methods"',
            str(failure.exception)
        )

    def test_dict_on_invalid_key(self):
        test_yaml = {'tests': [{
            'name': '...',
            'GET': '/',
            'response_html': {
                'foo': 'hello',
                'bar': 'world',
            }
        }]}

        with self.assertRaises(exception.GabbiFormatError) as failure:
            suitemaker.test_suite_from_dict(self.loader, 'foo', test_yaml, '.',
                                            'localhost', 80, None, None)
        self.assertIn(
            "invalid key in test: 'response_html'",
            str(failure.exception)
        )



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test the works of inner and outer fixtures.

An "outer" fixture runs once per test suite. An "inner" is per test request.
"""

import os
import sys

import fixtures

from gabbi import driver
from gabbi import fixture
from gabbi.tests import simple_wsgi


TESTS_DIR = 'gabbits_inner'
COUNT_INNER = 0
COUNT_OUTER = 0


class OuterFixture(fixture.GabbiFixture):
    """Assert an outer fixture is only started once and is stopped."""

    def start_fixture(self):
        global COUNT_OUTER
        COUNT_OUTER += 1

    def stop_fixture(self):
        assert COUNT_OUTER == 1


class InnerFixture(fixtures.Fixture):
    """Test that setUp is called 3 times."""

    def setUp(self):
        super(InnerFixture, self).setUp()
        global COUNT_INNER
        COUNT_INNER += 1

    def cleanUp(self):
        super(InnerFixture, self).cleanUp()
        assert 1 <= COUNT_INNER <= 3


def load_tests(loader, tests, pattern):
    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
    return driver.build_tests(test_dir, loader, host=None,
                              intercept=simple_wsgi.SimpleWsgi,
                              fixture_module=sys.modules[__name__],
                              inner_fixtures=[InnerFixture],
                              test_loader_name=__name__)



"""A sample pytest module."""

# For pathname munging
import os

# The module that build_tests comes from.
from gabbi import driver

# We need access to the WSGI application that hosts our service
from myapp import wsgiapp

# We're using fixtures in the YAML files, we need to know where to
# load them from.
from myapp.test import fixtures

# By convention the YAML files are put in a directory named
# "gabbits" that is in the same directory as the Python test file.
TESTS_DIR = 'gabbits'


def test_gabbits():
    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
    # Pass "require_ssl=True" as an argument to force all tests
    # to use SSL in requests.
    test_generator = driver.py_test_generator(
        test_dir, intercept=wsgiapp.app,
        fixture_module=fixtures)

    for test in test_generator:
        yield test



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""JSON-related content handling."""

import json

import six

from gabbi.handlers import base
from gabbi import json_parser


class JSONHandler(base.ContentHandler):
    """A ContentHandler for JSON

    * Structured test ``data`` is turned into JSON when request
      content-type is JSON.
    * Response bodies that are JSON strings are made into Python
      data on the test ``response_data`` attribute when the response
      content-type is JSON.
    * A ``response_json_paths`` response handler is added.
    * JSONPaths in $RESPONSE substitutions are supported.
    """

    test_key_suffix = 'json_paths'
    test_key_value = {}

    @staticmethod
    def accepts(content_type):
        content_type = content_type.split(';', 1)[0].strip()
        return (content_type.endswith('+json') or
                content_type.startswith('application/json'))

    @classmethod
    def replacer(cls, response_data, match):
        return cls.extract_json_path_value(response_data, match)

    @staticmethod
    def dumps(data, pretty=False, test=None):
        if pretty:
            return json.dumps(data, indent=2, separators=(',', ': '))
        else:
            return json.dumps(data)

    @staticmethod
    def loads(data):
        return json.loads(data)

    @staticmethod
    def extract_json_path_value(data, path):
        """Extract the value at JSON Path path from the data.

        The input data is a Python datastructure, not a JSON string.
        """
        path_expr = json_parser.parse(path)
        matches = [match.value for match in path_expr.find(data)]
        if matches:
            if len(matches) > 1:
                return matches
            else:
                return matches[0]
        else:
            raise ValueError(
                "JSONPath '%s' failed to match on data: '%s'" % (path, data))

    def action(self, test, path, value=None):
        """Test json_paths against json data."""
        # Do template expansion in the left hand side.
        path = test.replace_template(path)
        try:
            match = self.extract_json_path_value(
                test.response_data, path)
        except AttributeError:
            raise AssertionError('unable to extract JSON from test results')
        except ValueError:
            raise AssertionError('json path %s cannot match %s' %
                                 (path, test.response_data))

        # read data from disk if the value starts with '<@'
        if isinstance(value, str) and value.startswith('<@'):
            info = test.load_data_file(value.replace('<@', '', 1))
            info = six.text_type(info, 'UTF-8')
            value = self.loads(info)

        # If expected is a string, check to see if it is a regex.
        is_regex = (isinstance(value, six.string_types) and
                    value.startswith('/') and
                    value.endswith('/') and
                    len(value) > 1)
        expected = test.replace_template(value, escape_regex=is_regex)
        if is_regex:
            expected = expected[1:-1]
            # match may be a number so stringify
            match = six.text_type(match)
            test.assertRegexpMatches(
                match, expected,
                'Expect jsonpath %s to match /%s/, got %s' %
                (path, expected, match))
        else:
            test.assertEqual(expected, match,
                             'Unable to match %s as %s, got %s' %
                             (path, expected, match))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test loading data from files with <@.
"""

import unittest

from gabbi import case
from six.moves import mock


@mock.patch(
    'gabbi.case.open',
    new_callable=mock.mock_open,
    read_data='dummy content',
    create=True,
)
class DataFileTest(unittest.TestCase):
    """Reading from local file is only allowed at or below the
    test_directory level.
    """

    def setUp(self):
        self.http_case = case.HTTPTestCase('test_request')

    def _assert_content_read(self, filepath):
        self.assertEqual(
            'dummy content', self.http_case.load_data_file(filepath))

    def test_load_file(self, m_open):
        self.http_case.test_directory = '.'
        self._assert_content_read('data.json')
        m_open.assert_called_with('./data.json', mode='rb')

    def test_load_file_in_directory(self, m_open):
        self.http_case.test_directory = '.'
        self._assert_content_read('a/b/c/data.json')
        m_open.assert_called_with('./a/b/c/data.json', mode='rb')

    def test_load_file_in_root(self, m_open):
        self.http_case.test_directory = '.'
        filepath = '/top-level.private'

        with self.assertRaises(ValueError):
            self.http_case.load_data_file(filepath)
        self.assertFalse(m_open.called)

    def test_load_file_in_parent_dir(self, m_open):
        self.http_case.test_directory = '.'
        filepath = '../file-in-parent-dir.txt'

        with self.assertRaises(ValueError):
            self.http_case.load_data_file(filepath)
        self.assertFalse(m_open.called)

    def test_load_file_within_test_directory(self, m_open):
        self.http_case.test_directory = '/a/b/c'
        self._assert_content_read('../../b/c/file-in-test-dir.txt')
        m_open.assert_called_with(
            '/a/b/c/../../b/c/file-in-test-dir.txt', mode='rb')

    def test_load_file_not_within_test_directory(self, m_open):
        self.http_case.test_directory = '/a/b/c'
        filepath = '../../b/E/file-in-test-dir.txt'
        with self.assertRaises(ValueError):
            self.http_case.load_data_file(filepath)
        self.assertFalse(m_open.called)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test History Replacer.
"""

import unittest

from gabbi import case
from gabbi.handlers import jsonhandler
from gabbi import suitemaker


class HistoryTest(unittest.TestCase):
    """Test history variable."""

    def setUp(self):
        super(HistoryTest, self).setUp()
        self.test_class = case.HTTPTestCase
        self.test = suitemaker.TestBuilder('mytest', (self.test_class,),
                                           {'test_data': {},
                                            'content_handlers': [],
                                            'history': {},
                                            })

    def test_header_replace_prior(self):
        self.test.test_data = '$HEADERS["content-type"]'
        self.test.response = {'content-type': 'test_content'}
        self.test.prior = self.test

        header = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test_content', header)

    def test_header_replace_with_history(self):
        self.test.test_data = '$HISTORY["mytest"].$HEADERS["content-type"]'
        self.test.response = {'content-type': 'test_content'}
        self.test.history["mytest"] = self.test

        header = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test_content', header)

    def test_header_replace_with_history_regex(self):
        self.test.test_data = '/$HISTORY["mytest"].$HEADERS["content-type"]/'
        self.test.response = {'content-type': 'test+content'}
        self.test.history["mytest"] = self.test

        header = self.test('test_request').replace_template(
            self.test.test_data, escape_regex=True)
        self.assertEqual(r'/test\+content/', header)

    def test_response_replace_prior(self):
        self.test.test_data = '$RESPONSE["$.object.name"]'
        json_handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.content_handlers = [json_handler]
        self.test.prior = self.test
        self.test.response = {'content-type': 'application/json'}
        self.test.response_data = {
            'object': {'name': 'test history'}
        }

        response = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test history', response)

    def test_response_replace_prior_regex(self):
        self.test.test_data = '/$RESPONSE["$.object.name"]/'
        json_handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.content_handlers = [json_handler]
        self.test.prior = self.test
        self.test.response = {'content-type': 'application/json'}
        self.test.response_data = {
            'object': {'name': 'test history.'}
        }

        response = self.test('test_request').replace_template(
            self.test.test_data, escape_regex=True)
        self.assertEqual(r'/test\ history\./', response)

    def test_response_replace_with_history(self):
        self.test.test_data = '$HISTORY["mytest"].$RESPONSE["$.object.name"]'
        json_handler = jsonhandler.JSONHandler()
        self.test.content_type = "application/json"
        self.test.content_handlers = [json_handler]
        self.test.history["mytest"] = self.test
        self.test.response = {'content-type': 'application/json'}
        self.test.response_data = {
            'object': {'name': 'test history'}
        }

        response = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test history', response)

    def test_cookie_replace_prior(self):
        self.test.test_data = '$COOKIE'
        self.test.response = {'set-cookie': 'test=cookie'}
        self.test.prior = self.test

        cookie = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test=cookie', cookie)

    def test_cookie_replace_prior_regex(self):
        self.test.test_data = '/$COOKIE/'
        self.test.response = {'set-cookie': 'test=cookie?'}
        self.test.prior = self.test

        cookie = self.test('test_request').replace_template(
            self.test.test_data, escape_regex=True)
        self.assertEqual(r'/test\=cookie\?/', cookie)

    def test_cookie_replace_history(self):
        self.test.test_data = '$HISTORY["mytest"].$COOKIE'
        self.test.response = {'set-cookie': 'test=cookie'}
        self.test.history["mytest"] = self.test

        cookie = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test=cookie', cookie)

    def test_location_replace_prior(self):
        self.test.test_data = '$LOCATION'
        self.test.location = 'test_location'
        self.test.prior = self.test

        location = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test_location', location)

    def test_location_replace_prior_regex(self):
        self.test.test_data = '/$LOCATION/'
        self.test.location = '..'
        self.test.prior = self.test

        location = self.test('test_request').replace_template(
            self.test.test_data, escape_regex=True)
        self.assertEqual(r'/\.\./', location)

    def test_location_replace_history(self):
        self.test.test_data = '$HISTORY["mytest"].$LOCATION'
        self.test.location = 'test_location'
        self.test.history["mytest"] = self.test

        location = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test_location', location)

    def test_url_replace_prior(self):
        self.test.test_data = '$URL'
        self.test.url = 'test_url'
        self.test.prior = self.test

        url = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test_url', url)

    def test_url_replace_prior_regex(self):
        self.test.test_data = '/$URL/'
        self.test.url = 'testurl?query'
        self.test.prior = self.test

        url = self.test('test_request').replace_template(
            self.test.test_data, escape_regex=True)
        self.assertEqual(r'/testurl\?query/', url)

    def test_url_replace_history(self):
        self.test.test_data = '$HISTORY["mytest"].$URL'
        self.test.url = 'test_url'
        self.test.history["mytest"] = self.test

        url = self.test('test_request').replace_template(
            self.test.test_data)
        self.assertEqual('test_url', url)


if __name__ == '__main__':
    unittest.main()



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""A pytest plugin that runs under the covers with gabbi.

This is a backwards compatible improvement to the way gabbi can work with
pytest. Tests are loaded (and yielded in the same way) but they are filtered
more correctly, and fixture start and stops are done more correctly.

This allows the test count to be accurate, which is nice.
"""

import pytest


# Globals storing the test-like functions to be used when starting
# and stopping a suite.
STARTS = {}
STOPS = {}


def get_cleanname(item):
    """Extract a test name from a pytest Function item."""
    if '[' in item.name:
        cleanname = item.name.split('[', 1)[1]
        cleanname = cleanname.split(']', 1)[0]
        return cleanname
    return item.name


def get_suitename(name):
    """Extract a test suite from a clean name.

    This is fragile. It assumes there are no underscores in
    suite names, which is not always true.
    """
    if name.startswith('start_') or name.startswith('stop_'):
        _, name = name.split('_', 1)
    return name.split('_', 2)[1]


def c_pytest_collection_modifyitems(items, config):
    """Set the starters and stoppers for a limited collection of tests."""
    latest_suite = None
    latest_item = None
    for item in items:
        cleanname = get_cleanname(item)
        if not cleanname.startswith(
                ('stop_driver_', 'start_driver_', 'driver_')):
            continue
        prefix, testname = cleanname.split('_', 1)
        suitename, _ = testname.split('_', 1)
        if prefix == 'start' or prefix == 'stop':
            continue
        if latest_suite != suitename:
            item.starter = STARTS[suitename]
            if latest_item:
                latest_item.stopper = STOPS[
                    get_suitename(get_cleanname(latest_item))]
        latest_suite = suitename
        latest_item = item
    # Set the last stopper in the list
    if latest_item:
        latest_item.stopper = STOPS[get_suitename(get_cleanname(latest_item))]


def a_pytest_collection_modifyitems(items, config):
    """Traverse collected tests to save START and STOPS.

    Remove those START and STOPS from the tests to run.
    """
    remaining = []
    deselected = []
    for item in items:
        cleanname = get_cleanname(item)
        if not cleanname.startswith(
                ('stop_driver_', 'start_driver_', 'driver_')):
            remaining.append(item)
            continue
        suitename = get_suitename(cleanname)
        if cleanname.startswith('start_'):
            test = item.callspec.params['test']
            result = item.callspec.params['result']
            STARTS[suitename] = (test, result)
            deselected.append(item)
        elif cleanname.startswith('stop_'):
            test = item.callspec.params['test']
            STOPS[suitename] = test
            deselected.append(item)
        else:
            remaining.append(item)

    if deselected:
        items[:] = remaining


@pytest.hookimpl(hookwrapper=True)
def pytest_collection_modifyitems(items, config):
    """Hook for processing collected tests.

    Discover start and stops, then use the default hook
    for filter for keywords and markers, then attach
    starter and stopper to the remaining tests.
    """
    a_pytest_collection_modifyitems(items, config)
    yield
    c_pytest_collection_modifyitems(items, config)


def pytest_runtest_setup(item):
    """Run a starter if a test has one.

    This is done before run, so it means that a single test will
    run its priors after running this.
    """
    if hasattr(item, 'starter'):
        test, result = item.starter
        test(result)


def pytest_runtest_teardown(item, nextitem):
    """Run a stopper if a test has one."""
    if hasattr(item, 'stopper'):
        item.stopper()



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test use_prior_test directive.
"""

import copy
from six.moves import mock
import unittest

from gabbi import case


class UsePriorTest(unittest.TestCase):

    @staticmethod
    def make_test_case(use_prior_test=None):
        http_case = case.HTTPTestCase('test_request')
        http_case.test_data = copy.copy(case.BASE_TEST)
        if use_prior_test is not None:
            http_case.test_data['use_prior_test'] = use_prior_test
        return http_case

    @mock.patch('gabbi.case.HTTPTestCase._run_test')
    def test_use_prior_true(self, m_run_test):
        http_case = self.make_test_case(True)
        http_case.has_run = False
        http_case.prior = self.make_test_case(True)
        http_case.prior.run = mock.MagicMock(unsafe=True)
        http_case.prior.has_run = False

        http_case.test_request()
        http_case.prior.run.assert_called_once()

    @mock.patch('gabbi.case.HTTPTestCase._run_test')
    def test_use_prior_false(self, m_run_test):
        http_case = self.make_test_case(False)
        http_case.has_run = False
        http_case.prior = self.make_test_case(True)
        http_case.prior.run = mock.MagicMock(unsafe=True)
        http_case.prior.has_run = False

        http_case.test_request()
        http_case.prior.run.assert_not_called()

    @mock.patch('gabbi.case.HTTPTestCase._run_test')
    def test_use_prior_default_true(self, m_run_test):
        http_case = self.make_test_case()
        http_case.has_run = False
        http_case.prior = self.make_test_case(True)
        http_case.prior.run = mock.MagicMock(unsafe=True)
        http_case.prior.has_run = False

        http_case.test_request()
        http_case.prior.run.assert_called_once()



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Core response handlers."""

from gabbi.handlers import base


class StringResponseHandler(base.ResponseHandler):
    """Test for matching strings in the the response body."""

    test_key_suffix = 'strings'
    test_key_value = []

    def action(self, test, expected, value=None):
        expected = test.replace_template(expected)
        test.assert_in_or_print_output(expected, test.output)


class ForbiddenHeadersResponseHandler(base.ResponseHandler):
    """Test that listed headers are not in the response."""

    test_key_suffix = 'forbidden_headers'
    test_key_value = []

    def action(self, test, forbidden, value=None):
        # normalize forbidden header to lower case
        forbidden = test.replace_template(forbidden).lower()
        test.assertNotIn(forbidden, test.response,
                         'Forbidden header %s found in response' % forbidden)


class HeadersResponseHandler(base.ResponseHandler):
    """Compare expected headers with actual headers.

    If a header value is wrapped in ``/`` it is treated as a raw
    regular expression.

    Headers values are always treated as strings.
    """

    test_key_suffix = 'headers'
    test_key_value = {}

    def action(self, test, header, value=None):
        header = header.lower()  # case-insensitive comparison
        response = test.response

        header_value = str(value)
        is_regex = (header_value.startswith('/') and
                    header_value.endswith('/') and
                    len(header_value) > 1)
        header_value = test.replace_template(header_value,
                                             escape_regex=is_regex)

        try:
            response_value = str(response[header])
        except KeyError:
            raise AssertionError(
                "'%s' header not present in response: %s" % (
                    header, response.keys()))

        if is_regex:
            header_value = header_value[1:-1]
            test.assertRegex(
                response_value, header_value,
                'Expect header %s to match /%s/, got %s' %
                (header, header_value, response_value))
        else:
            test.assertEqual(header_value, response_value,
                             'Expect header %s with value %s, got %s' %
                             (header, header_value, response[header]))



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Utility methods shared by some tests."""

import os


def set_test_environ():
    """Set some environment variables used in tests."""
    os.environ['GABBI_TEST_URL'] = 'takingnames'

    # Setup environment variables for `coerce.yaml`
    os.environ['ONE'] = '1'
    os.environ['DECIMAL'] = '1.0'
    os.environ['ARRAY_STRING'] = '[1,2,3]'
    os.environ['TRUE'] = 'true'
    os.environ['FALSE'] = 'false'
    os.environ['STRING'] = 'val'
    os.environ['NULL'] = 'null'



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Base classes for response and content handlers."""


from gabbi.exception import GabbiFormatError


class ResponseHandler(object):
    """Add functionality for making assertions about an HTTP response.

    A subclass may implement two methods: ``action`` and ``preprocess``.

    ``preprocess`` takes one argument, the ``TestCase``. It is called exactly
    once for each test before looping across the assertions. It is used,
    rarely, to copy the ``test.output`` into a useful form (such as a parsed
    DOM).

    ``action`` takes two or three arguments. If ``test_key_value`` is a list
    ``action`` is called with the test case and a single list item. If
    ``test_key_value`` is a dict then ``action`` is called with the test case
    and a key and value pair.
    """

    test_key_suffix = ''
    test_key_value = []

    def __init__(self):
        self._register()

    def __call__(self, test):
        if test.test_data[self._key]:
            self.preprocess(test)
            if type(self.test_key_value) != type(test.test_data[self._key]):
                raise GabbiFormatError(
                    "%s in '%s' has incorrect type, must be %s"
                    % (self._key, test.test_data['name'],
                       type(self.test_key_value)))
            for item in test.test_data[self._key]:
                try:
                    value = test.test_data[self._key][item]
                except (TypeError, KeyError):
                    value = None
                self.action(test, item, value=value)

    def preprocess(self, test):
        """Do any pre-single-test preprocessing."""
        pass

    def action(self, test, item, value=None):
        """Test an individual entry for this response handler.

        If the entry is a key value pair the key is in item and the
        value in value. Otherwise the entry is considered a single item
        from a list.
        """
        pass

    def _register(self):
        """Register this handler on the provided test class."""
        self.response_handler = None
        self.content_handler = None
        if self.test_key_suffix:
            self._key = 'response_%s' % self.test_key_suffix
            self.test_base = {self._key: self.test_key_value}
            self.response_handler = self
        if hasattr(self, 'accepts'):
            self.content_handler = self

    def __eq__(self, other):
        if isinstance(other, ResponseHandler):
            return self.__class__ == other.__class__
        return False

    def __ne__(self, other):
        return not self.__eq__(other)


class ContentHandler(ResponseHandler):
    """A subclass of ResponseHandlers that adds content handling."""

    @staticmethod
    def accepts(content_type):
        """Return True if this handler can handler this type."""
        return False

    @classmethod
    def replacer(cls, response_data, path):
        """Return the string that is replacing RESPONSE."""
        return path

    @staticmethod
    def dumps(data, pretty=False, test=None):
        """Return structured data as a string.

        If pretty is true, prettify.
        """
        return data

    @staticmethod
    def loads(data):
        """Create structured (Python) data from a stream."""
        return data



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test pytest driving of tests.

Unittest loaders don't see this file and pytest doesn't see load_tests,
so we manage to get coverage across both types of drivers, from tox,
without duplication.
"""

import os

from gabbi import driver
# TODO(cdent): this test_* needs to be imported bare or things do not work
from gabbi.driver import test_pytest  # noqa
from gabbi.tests import simple_wsgi
from gabbi.tests import test_intercept
from gabbi.tests import util

TESTS_DIR = 'gabbits_intercept'


def pytest_generate_tests(metafunc):
    util.set_test_environ()
    test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)
    driver.py_test_generator(
        test_dir, intercept=simple_wsgi.SimpleWsgi,
        fixture_module=test_intercept,
        response_handlers=[test_intercept.TestResponseHandler],
        metafunc=metafunc)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Utility functions grab bag."""

import io
import os

import six
import yaml


try:  # Python 3
    ConnectionRefused = ConnectionRefusedError
except NameError:  # Python 2
    import socket
    ConnectionRefused = socket.error


import colorama
from six.moves.urllib import parse as urlparse


def create_url(base_url, host, port=None, prefix='', ssl=False):
    """Given pieces of a path-based url, return a fully qualified url."""
    scheme = 'http'

    # A host with : in it at this stage is assumed to be an IPv6
    # address of some kind (they come in many forms). Port should
    # already have been stripped off.
    if ':' in host and not (host.startswith('[') and host.endswith(']')):
        host = '[%s]' % host

    if port and not _port_follows_standard(port, ssl):
        netloc = '%s:%s' % (host, port)
    else:
        netloc = host

    if ssl:
        scheme = 'https'

    parsed_url = urlparse.urlsplit(base_url)
    query_string = parsed_url.query
    path = parsed_url.path

    # Guard against a prefix of None or the url already having the
    # prefix. Without the startswith check, the tests in prefix.yaml
    # fail. This is a pragmatic fix which does this for any URL in a
    # test request that does not have a scheme and does not
    # distinguish between URLs in a gabbi test file and those
    # generated by the server. Idealy we would not mutate nor need
    # to check URLs returned from the server. Doing that, however,
    # would require more complex data handling than we have now and
    # this covers most common cases and will be okay until someone
    # reports a bug.
    if prefix and not path.startswith(prefix):
        prefix = prefix.rstrip('/')
        path = path.lstrip('/')
        path = '%s/%s' % (prefix, path)

    return urlparse.urlunsplit((scheme, netloc, path, query_string, ''))


def decode_response_content(header_dict, content):
    """Decode content to a proper string."""
    content_type, charset = extract_content_type(header_dict)

    if not_binary(content_type) and isinstance(content, six.binary_type):
        return content.decode(charset)
    else:
        return content


def extract_content_type(header_dict, default='application/binary'):
    """Extract parsed content-type from headers."""
    content_type = header_dict.get('content-type',
                                   default).strip().lower()
    return parse_content_type(content_type)


def get_colorizer(stream):
    """Return a function to colorize a string.

    Only if stream is a tty .
    """
    if stream.isatty() or os.environ.get('GABBI_FORCE_COLOR', False):
        colorama.init()
        return _colorize
    else:
        return lambda x, y: y


def load_yaml(handle=None, yaml_file=None):
    """Read and parse any YAML file or filehandle.

    Let exceptions flow where they may.

    If no file or handle is provided, read from STDIN.
    """
    if yaml_file:
        with io.open(yaml_file, encoding='utf-8') as source:
            return yaml.safe_load(source.read())

    # This will intentionally raise AttributeError if handle is None.
    return yaml.safe_load(handle.read())


def not_binary(content_type):
    """Decide if something is content we'd like to treat as a string."""
    return (content_type.startswith('text/') or
            content_type.endswith('+xml') or
            content_type.endswith('+json') or
            content_type == 'application/javascript' or
            content_type.startswith('application/json') or
            content_type.startswith('application/xml'))


def parse_content_type(content_type, default_charset='utf-8'):
    """Parse content type value for media type and charset."""
    charset = default_charset
    if ';' in content_type:
        content_type, parameter_strings = (attr.strip() for attr
                                           in content_type.split(';', 1))
        try:
            parameter_pairs = [atom.strip().split('=')
                               for atom in parameter_strings.split(';')]
            parameters = {name: value for name, value in parameter_pairs}
            charset = parameters['charset']
        except (ValueError, KeyError):
            # KeyError when no charset found.
            # ValueError when the parameter_strings are poorly
            # formed (for example trailing ;)
            pass

    return (content_type, charset)


def host_info_from_target(target, prefix=None):
    """Turn url or host:port and target into test destination."""
    force_ssl = False
    split_url = urlparse.urlparse(target)

    if split_url.scheme:
        if split_url.scheme == 'https':
            force_ssl = True
        return split_url.hostname, split_url.port, split_url.path, force_ssl
    else:
        target = target
        prefix = prefix

    if ':' in target and '[' not in target:
        host, port = target.rsplit(':', 1)
    elif ']:' in target:
        host, port = target.rsplit(':', 1)
    else:
        host = target
        port = None
    host = host.replace('[', '').replace(']', '')

    return host, port, prefix, force_ssl


def _colorize(color, message):
    """Add a color to the message."""
    try:
        return getattr(colorama.Fore, color) + message + colorama.Fore.RESET
    except AttributeError:
        return message


def _port_follows_standard(port, ssl):
    """Return True if a standard port is using a non-standard ssl setting."""
    port = int(port)
    return (port == 443 and ssl) or (port == 80 and not ssl)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Test that the driver warns on bad yaml name."""

import os
import unittest
import warnings

from gabbi import driver
from gabbi import exception


TESTS_DIR = 'warning_gabbits'


class DriverTest(unittest.TestCase):

    def setUp(self):
        super(DriverTest, self).setUp()
        self.loader = unittest.defaultTestLoader
        self.test_dir = os.path.join(os.path.dirname(__file__), TESTS_DIR)

    def test_driver_warnings_on_files(self):
        with warnings.catch_warnings(record=True) as the_warnings:
            driver.build_tests(
                self.test_dir, self.loader, host='localhost', port=8001)
            self.assertEqual(1, len(the_warnings))
            the_warning = the_warnings[-1]
            self.assertEqual(
                the_warning.category, exception.GabbiSyntaxWarning)
            self.assertIn("'_' in test filename", str(the_warning.message))
